@@ -20,9 +20,10 @@ class Contact:
 	'''Information concerning each contact'''
 	def __init__(self, jid='', name='', groups=[], show='', status='', sub='',
 	ask='', resource='', priority=0, keyID='', our_chatstate=None,
-	chatstate=None, last_status_time=None, msg_id = None, composing_jep = None):
+	chatstate=None, last_status_time=None, msg_id = None, composing_xep = None):
 		self.jid = jid
 		self.name = name
+		self.contact_name = '' # nick choosen by contact
 		self.groups = groups
 		self.show = show
 		self.status = status
@@ -42,9 +43,9 @@ class Contact:
 		self.our_chatstate = our_chatstate
 		self.msg_id = msg_id
 		# tell which JEP we're using for composing state
-		# None = have to ask, JEP-0022 = use this jep,
-		# JEP-0085 = use this jep, False = no composing support
-		self.composing_jep = composing_jep
+		# None = have to ask, XEP-0022 = use this jep,
+		# XEP-0085 = use this jep, False = no composing support
+		self.composing_xep = composing_xep
 		# this is contact's chatstate
 		self.chatstate = chatstate
 		self.last_status_time = last_status_time
@@ -57,6 +58,8 @@ class Contact:
 	def get_shown_name(self):
 		if self.name:
 			return self.name
+		if self.contact_name:
+			return self.contact_name
 		return self.jid.split('@')[0]
 
 	def is_hidden_from_roster(self):
@@ -141,10 +144,10 @@ class Contacts:
 
 	def create_contact(self, jid='', name='', groups=[], show='', status='',
 		sub='', ask='', resource='', priority=0, keyID='', our_chatstate=None,
-		chatstate=None, last_status_time=None, composing_jep=None):
+		chatstate=None, last_status_time=None, composing_xep=None):
 		return Contact(jid, name, groups, show, status, sub, ask, resource,
 			priority, keyID, our_chatstate, chatstate, last_status_time,
-			composing_jep)
+			composing_xep)
 	
 	def copy_contact(self, contact):
 		return self.create_contact(jid = contact.jid, name = contact.name,
@@ -186,6 +189,9 @@ class Contacts:
 		if len(self._contacts[account][contact.jid]) == 0:
 			del self._contacts[account][contact.jid]
 
+	def clear_contacts(self, account):
+		self._contacts[account] = {}
+
 	def remove_jid(self, account, jid):
 		'''Removes all contacts for a given jid'''
 		if not self._contacts.has_key(account):
@@ -264,18 +270,24 @@ class Contacts:
 				if jid == our_jid:
 					continue
 				if common.gajim.jid_is_transport(jid) and not \
-				common.gajim.config.get('show_transports_group'):
+					_('Transports') in groups:
 					# do not count transports
 					continue
 				contact = self.get_contact_with_highest_priority(account, jid)
+				if _('Not in roster') in contact.groups:
+					continue
 				in_groups = False
 				if groups == []:
 					in_groups = True
 				else:
 					contact_groups = contact.groups
 					if not contact_groups:
-						# Contact is not in a group, so count it in General group
-						contact_groups.append(_('General'))
+						# Contact is not in a group, so count it in General or
+						# Transports group
+						if common.gajim.jid_is_transport(jid):
+							contact_groups = [_('Transports')]
+						else:
+							contact_groups = [_('General')]
 					for group in groups:
 						if group in contact_groups:
 							in_groups = True
@@ -373,45 +385,71 @@ class Contacts:
 					answers.append(data)
 		return answers
 
-	def _get_data_score(self, data):
-		'''compute thescore of a gived data
-		data is {'jid': jid, 'account': account, 'order': order}
-		order is optional
-		score = (max_order - order)*10000 + is_jabber*priority*10 + status'''
-		jid = data['jid']
-		account = data['account']
-		max_order = 0
-		order = 0
-		if data.has_key('order'):
-			order = data['order']
-		if order:
-			family = self.get_metacontacts_family(account, jid)
-			for data_ in family:
-				if data_.has_key('order') and data_['order'] > max_order:
-					max_order = data_['order']
-		contact = self.get_contact_with_highest_priority(account, jid)
-		score = (max_order - order)*10000
-		
-		if common.gajim.get_transport_name_from_jid(jid) is None and \
-		contact.show not in ('error', 'offline'):
-			score += 10
-			if contact.priority > 0:
-				score += contact.priority * 10
-		score += ['not in roster', 'error', 'offline', 'invisible', 'dnd', 'xa',
-		'away', 'chat', 'online', 'requested', 'message'].index(contact.show)
-		return score
+	def compare_metacontacts(self, data1, data2):
+		'''compare 2 metacontacts.
+		Data is {'jid': jid, 'account': account, 'order': order}
+		order is optional'''
+		jid1 = data1['jid']
+		jid2 = data2['jid']
+		account1 = data1['account']
+		account2 = data2['account']
+		contact1 = self.get_contact_with_highest_priority(account1, jid1)
+		contact2 = self.get_contact_with_highest_priority(account2, jid2)
+		show_list = ['not in roster', 'error', 'offline', 'invisible', 'dnd',
+			'xa', 'away', 'chat', 'online', 'requested', 'message']
+		# contact can be null when we fill the roster on connection
+		if not contact1:
+			show1 = 0
+			priority1 = 0
+		else:
+			show1 = show_list.index(contact1.show)
+			priority1 = contact1.priority
+		if not contact2:
+			show2 = 0
+			priority2 = 0
+		else:
+			show2 = show_list.index(contact2.show)
+			priority2 = contact2.priority
+		# If only one is offline, it's always second
+		if show1 > 2 and show2 < 3:
+			return 1
+		if show2 > 2 and show1 < 3:
+			return -1
+		if 'order' in data1 and 'order' in data2:
+			if data1['order'] > data2['order']:
+				return 1
+			if data1['order'] < data2['order']:
+				return -1
+		transport1 = common.gajim.get_transport_name_from_jid(jid1)
+		transport2 = common.gajim.get_transport_name_from_jid(jid2)
+		if transport2 and not transport1:
+			return 1
+		if transport1 and not transport2:
+			return -1
+		if priority1 > priority2:
+			return 1
+		if priority2 > priority1:
+			return -1
+		if show1 > show2:
+			return 1
+		if show2 > show1:
+			return -1
+		if jid1 > jid2:
+			return 1
+		if jid2 > jid1:
+			return -1
+		# If all is the same, compare accounts, they can't be the same
+		if account1 > account2:
+			return 1
+		if account2 > account1:
+			return -1
+		return 0
 
 	def get_metacontacts_big_brother(self, family):
 		'''which of the family will be the big brother under wich all
 		others will be ?'''
-		max_score = 0
-		max_data = family[0]
-		for data in family:
-			score = self._get_data_score(data)
-			if score > max_score:
-				max_score = score
-				max_data = data
-		return max_data
+		family.sort(cmp=self.compare_metacontacts)
+		return family[-1]
 
 	def is_pm_from_jid(self, account, jid):
 		'''Returns True if the given jid is a private message jid'''
