@@ -209,12 +209,9 @@ class HistoryWindow:
 		end_iter = buf.get_end_iter()
 		
 		if gajim.config.get('print_time') == 'always':
-			before_str = gajim.config.get('before_time')
-			before_str = helpers.from_one_line(before_str)
-			after_str = gajim.config.get('after_time')
-			after_str = helpers.from_one_line(after_str)
-			format = before_str + '%X' + after_str + ' '
-			tim = time.strftime(format, time.localtime(float(tim)))
+			timestamp_str = gajim.config.get('time_stamp')
+			timestamp_str = helpers.from_one_line(timestamp_str)
+			tim = time.strftime(timestamp_str, time.localtime(float(tim)))
 			buf.insert(end_iter, tim) # add time
 		elif gajim.config.get('print_time') == 'sometimes':
 			every_foo_seconds = 60 * gajim.config.get(
@@ -300,6 +297,9 @@ class HistoryWindow:
 		the True/False thingy depending if it's expanded or not
 		this function is called in a timeout just after expanded state changes'''
 		parent = widget.get_parent() # vbox
+		if not parent:
+			# Windows closed since we launch timeout
+			return
 		expanded = widget.get_expanded()
 		w, h = self.window.get_size()
 		if expanded: # resize to larger in height the window
